@@ -48,6 +48,7 @@ struct _LiPackageGraphPrivate
 	guint max_progress;
 
 	GHashTable *foundations;
+	gboolean ignore_foundations;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (LiPackageGraph, li_package_graph, G_TYPE_OBJECT)
@@ -90,6 +91,7 @@ li_package_graph_init (LiPackageGraph *pg)
 	priv->nindex = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	priv->install_todo = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 	priv->foundations = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	priv->ignore_foundations = FALSE;
 }
 
 /**
@@ -111,7 +113,7 @@ li_package_graph_update_foundations_table (LiPackageGraph *pg, GError **error)
 	if (g_file_query_exists (file, NULL)) {
 		li_config_data_load_file (fdconf, file, &tmp_error);
 	} else {
-		g_warning ("No foundation (system-component) was defined. Continuing without that knowledge.");
+		g_warning ("No foundations (system-components) were defined. Continuing without that knowledge.");
 	}
 	g_object_unref (file);
 	if (tmp_error != NULL) {
@@ -441,6 +443,35 @@ li_package_graph_node_get_any_parent_manual (LiPackageGraph *pg, LiPkgInfo *root
 	return FALSE;
 }
 
+/**
+ * li_package_graph_node_is_origin:
+ * @pg: An instance of #LiPackageGraph.
+ *
+ * Check if this node is an origin (it is no child of any other node).
+ */
+gboolean
+li_package_graph_node_is_origin (LiPackageGraph *pg, LiPkgInfo *root)
+{
+	guint i;
+	LiPackageGraphPrivate *priv = GET_PRIVATE (pg);
+
+	for (i = 0; i < priv->alist->len; i++) {
+		guint j;
+		GPtrArray *row = (GPtrArray*) g_ptr_array_index (priv->alist, i);
+
+		if (LI_PKG_INFO (g_ptr_array_index (row, 0)) == root)
+			continue;
+
+		for (j = 0; j < row->len; j++) {
+			LiPkgInfo *pki = LI_PKG_INFO (g_ptr_array_index (row, j));
+			if (pki == root)
+				return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 /**
  * li_package_graph_reset:
  *
@@ -492,6 +523,11 @@ li_package_graph_test_foundation_dependency (LiPackageGraph *pg, LiPkgInfo *dep_
 	if (!g_str_has_prefix (pkname, "foundation:"))
 		return FALSE;
 
+	if (priv->ignore_foundations) {
+		g_debug ("Ignoring dependency on foundation: %s", pkname);
+		return TRUE;
+	}
+
 	if (g_hash_table_lookup (priv->foundations, pkname) != NULL) {
 		/* foundation was found, dependency is satisfied! */
 		g_debug ("Detected system dependency '%s' as satisfied.", pkname);
@@ -505,6 +541,18 @@ li_package_graph_test_foundation_dependency (LiPackageGraph *pg, LiPkgInfo *dep_
 	}
 }
 
+/**
+ * li_package_graph_set_ignore_foundations:
+ * @pg: An instance of #LiPackageGraph
+ * @ignore: %TRUE if foundation dependencies should be ignored.
+ */
+void
+li_package_graph_set_ignore_foundations (LiPackageGraph *pg, gboolean ignore)
+{
+	LiPackageGraphPrivate *priv = GET_PRIVATE (pg);
+	priv->ignore_foundations = ignore;
+}
+
 /**
  * li_find_satisfying_pkg:
  *
