@@ -417,7 +417,7 @@ li_package_read_component_data (LiPackage *pkg, const gchar *data, GError **erro
 	/* do not filter languages */
 	as_metadata_set_locale (mdata, "ALL");
 
-	as_metadata_parse_data (mdata, data, &tmp_error);
+	as_metadata_parse_xml (mdata, data, &tmp_error);
 	priv->cpt = g_object_ref (as_metadata_get_component (mdata));
 	g_object_unref (mdata);
 	if (tmp_error != NULL) {
@@ -425,6 +425,10 @@ li_package_read_component_data (LiPackage *pkg, const gchar *data, GError **erro
 		return FALSE;
 	}
 
+	/* add information about which component kind we are dealing with to the local package metadata */
+	li_pkg_info_set_component_kind (priv->info,
+					as_component_kind_to_string (as_component_get_kind (priv->cpt)));
+
 	tmp = li_get_pkgname_from_component (priv->cpt);
 	if (tmp == NULL) {
 		g_set_error (error,
