@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009-2014 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2009-2016 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -31,10 +31,12 @@
 #include <appstream.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/utsname.h>
 #include <errno.h>
 #include "li-systemd-dbus.h"
 
+
 /**
  * SECTION:li-utils
  * @short_description: General-purpose helper functions for Limba
@@ -116,7 +118,11 @@ li_delete_dir_recursive (const gchar* dirname)
 		return TRUE;
 
 	dir = g_file_new_for_path (dirname);
-	enr = g_file_enumerate_children (dir, "standard::name", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &error);
+	enr = g_file_enumerate_children (dir,
+					 "standard::name",
+					 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+					 NULL,
+					 &error);
 	if (error != NULL)
 		goto out;
 
@@ -126,7 +132,7 @@ li_delete_dir_recursive (const gchar* dirname)
 	if (error != NULL)
 		goto out;
 	while (info != NULL) {
-		gchar *path;
+		g_autofree gchar *path = NULL;
 		path = g_build_filename (dirname, g_file_info_get_name (info), NULL);
 		if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
 			li_delete_dir_recursive (path);
@@ -158,7 +164,7 @@ out:
  * li_utils_find_files_matching:
  */
 GPtrArray*
-li_utils_find_files_matching (const gchar* dir, const gchar* pattern, gboolean recursive)
+li_utils_find_files_matching (const gchar *dir, const gchar *pattern, gboolean recursive)
 {
 	GPtrArray* list;
 	GError *error = NULL;
@@ -179,8 +185,8 @@ li_utils_find_files_matching (const gchar* dir, const gchar* pattern, gboolean r
 		if (g_file_info_get_is_hidden (file_info))
 			continue;
 		path = g_build_filename (dir,
-								 g_file_info_get_name (file_info),
-								 NULL);
+					 g_file_info_get_name (file_info),
+					 NULL);
 		if ((g_file_test (path, G_FILE_TEST_IS_DIR)) && (recursive)) {
 			GPtrArray *subdir_list;
 			guint i;
@@ -194,7 +200,7 @@ li_utils_find_files_matching (const gchar* dir, const gchar* pattern, gboolean r
 			}
 			for (i=0; i<subdir_list->len; i++)
 				g_ptr_array_add (list,
-								 g_strdup ((gchar *) g_ptr_array_index (subdir_list, i)));
+						 g_strdup ((gchar *) g_ptr_array_index (subdir_list, i)));
 			g_ptr_array_unref (subdir_list);
 		} else {
 			if (!li_str_empty (pattern)) {
@@ -418,17 +424,6 @@ li_get_uuid_string ()
 	return g_strndup (uuid_str, 36);
 }
 
-/**
- * li_get_install_root:
- *
- * A hack to support unit-tests running as non-root.
- */
-const gchar*
-li_get_software_root ()
-{
-	return LI_SOFTWARE_ROOT;
-}
-
 /**
  * li_get_current_arch_h:
  *
@@ -495,6 +490,43 @@ li_set_verbose_mode (gboolean verbose)
 	g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
 }
 
+/**
+ * li_reset_umask:
+ *
+ * Reset umask potentially set by the user to a default value,
+ * so we can create files with the correct permissions.
+ */
+void
+li_reset_umask (void)
+{
+	umask (0022);
+}
+
+/**
+ * li_get_pkgname_from_component:
+ *
+ * Convert an AppStream-ID into a valid bundle name.
+ */
+gchar*
+li_get_pkgname_from_component (AsComponent *cpt)
+{
+	gchar *pkgname;
+
+	pkgname = li_str_replace (as_component_get_id (cpt), ".desktop", "");
+	g_strstrip (pkgname);
+	if ((pkgname == NULL) || (g_strcmp0 (pkgname, "") == 0)) {
+		g_free (pkgname);
+		pkgname = li_str_replace (as_component_get_name (cpt), " ", "_");
+		if ((pkgname == NULL) || (g_strcmp0 (pkgname, "") == 0)) {
+			g_free (pkgname);
+			/* no package name is found, we give up */
+			return NULL;
+		}
+	}
+
+	return pkgname;
+}
+
 /**
  * li_compare_versions:
  *
