@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2014 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -19,9 +19,10 @@
  */
 
 #include <glib.h>
-#include "limba.h"
 #include <stdlib.h>
+#include "limba.h"
 
+#include "li-keyring.h"
 #include "li-pkg-cache.h"
 #include "li-utils-private.h"
 
@@ -35,15 +36,15 @@ test_remove_software ()
 
 	mgr = li_manager_new ();
 
-	li_manager_remove_software (mgr, "libfoo-1.0", &error);
+	li_manager_remove_software (mgr, "libfoo/1.0", &error);
 	g_assert_error (error, LI_MANAGER_ERROR, LI_MANAGER_ERROR_DEPENDENCY);
 	g_error_free (error);
 	error = NULL;
 
-	li_manager_remove_software (mgr, "foobar-1.0", &error);
+	li_manager_remove_software (mgr, "foobar/1.0", &error);
 	g_assert_no_error (error);
 
-	li_manager_remove_software (mgr, "libfoo-1.0", &error);
+	li_manager_remove_software (mgr, "libfoo/1.0", &error);
 	g_assert_no_error (error);
 
 	g_object_unref (mgr);
@@ -53,23 +54,44 @@ void
 test_installer_simple ()
 {
 	LiInstaller *inst;
+	LiManager *mgr;
 	g_autofree gchar *fname_app = NULL;
 	g_autofree gchar *fname_lib = NULL;
+	g_autofree gchar *gpg_key_fname = NULL;
+	LiTrustLevel tlevel;
 	GError *error = NULL;
 
 	fname_app = g_build_filename (datadir, "foobar.ipk", NULL);
 	fname_lib = g_build_filename (datadir, "libfoo.ipk", NULL);
+	gpg_key_fname = g_build_filename (datadir, "..", "gpg", "F90FD60F.gpg", NULL);
 
 	inst = li_installer_new ();
 
 	li_installer_open_file (inst, fname_app, &error);
 	g_assert_no_error (error);
+
+	/* trust level is none, since we don't know the key the package is signed with yet */
+	tlevel = li_installer_get_package_trust_level (inst, &error);
+	g_assert_error (error, LI_KEYRING_ERROR, LI_KEYRING_ERROR_KEY_MISSING);
+	g_assert (tlevel == LI_TRUST_LEVEL_LOW);
+	g_error_free (error);
+	error = NULL;
+
+	/* attempt an unsecure installation */
+	li_installer_set_allow_insecure (inst, TRUE);
 	li_installer_install (inst, &error);
+	li_installer_set_allow_insecure (inst, FALSE);
 	/* this has to fail, we don't have libfoo yet */
 	g_assert_error (error, LI_INSTALLER_ERROR, LI_INSTALLER_ERROR_DEPENDENCY_NOT_FOUND);
 	g_error_free (error);
 	error = NULL;
 
+	/* now trust the key the packages are signed with */
+	mgr = li_manager_new ();
+	li_manager_trust_key_file (mgr, gpg_key_fname, &error);
+	g_assert_no_error (error);
+	g_object_unref (mgr);
+
 	/* install library */
 	li_installer_open_file (inst, fname_lib, &error);
 	g_assert_no_error (error);
@@ -79,6 +101,11 @@ test_installer_simple ()
 	/* now we should be able to install the app */
 	li_installer_open_file (inst, fname_app, &error);
 	g_assert_no_error (error);
+	/* trust level should be MEDIUM now */
+	tlevel = li_installer_get_package_trust_level (inst, &error);
+	g_assert_no_error (error);
+	g_assert (tlevel == LI_TRUST_LEVEL_MEDIUM);
+
 	li_installer_install (inst, &error);
 	g_assert_no_error (error);
 
@@ -106,7 +133,7 @@ test_installer_embeddedpkg ()
 void
 test_manager ()
 {
-	GList *pkgs;
+	g_autoptr(GPtrArray) pkgs = NULL;
 	GError *error = NULL;
 	LiManager *mgr;
 
@@ -114,9 +141,7 @@ test_manager ()
 
 	pkgs = li_manager_get_software_list (mgr, &error);
 	g_assert_no_error (error);
-	g_assert (pkgs != NULL);
-
-	g_list_free (pkgs);
+	g_assert (pkgs->len > 0);
 
 	g_object_unref (mgr);
 }
@@ -179,7 +204,7 @@ test_install_from_repo ()
 
 	/* test an installation which fetches stuff from a remote location */
 	inst = li_installer_new ();
-	li_installer_open_remote (inst, "foobar-1.0", &error);
+	li_installer_open_remote (inst, "foobar/1.0", &error);
 	g_assert_no_error (error);
 
 	li_installer_install (inst, &error);
@@ -196,12 +221,12 @@ test_pkg_cache_setup ()
 
 	/* write sample repository file */
 	g_mkdir_with_parents ("/etc/limba/", 0755);
-	g_file_set_contents ("/etc/limba/sources.list", "# Limba Unit Tests\n\n# Test Repo\nhttp://people.freedesktop.org/~mak/stuff/limba-repo/\n", -1, &error);
+	g_file_set_contents ("/etc/limba/sources.list", "# Limba Unit Tests\n\n# Test Repo\ncommon,devel http://people.freedesktop.org/~mak/stuff/limba-repo/\n", -1, &error);
 	g_assert_no_error (error);
 
 	/* we need to trust the sample repository key */
 	mgr = li_manager_new ();
-	li_manager_receive_key (mgr, "D33A3F0CA16B0ACC51A60738494C8A5FBF4DECEB", &error);
+	li_manager_trust_key (mgr, "D33A3F0CA16B0ACC51A60738494C8A5FBF4DECEB", &error);
 	g_assert_no_error (error);
 	g_object_unref (mgr);
 
